package java.util.stream;

import java.util.function.DoubleConsumer;
import java.util.function.LongConsumer;
import java.util.PrimitiveIterator;
import java.util.function.IntConsumer;
import java.util.Objects;
import java.util.Spliterator;
import java.util.ArrayList;
import java.util.Spliterators;
import java.util.Iterator;
import java.util.function.IntFunction;
import java.util.Arrays;
import java.util.function.Consumer;

class SpinedBuffer<E> extends AbstractSpinedBuffer implements Consumer<E>, Iterable<E>
{
    protected E[] curChunk;
    protected E[][] spine;
    private static final int SPLITERATOR_CHARACTERISTICS = 16464;
    
    SpinedBuffer(int n) {
        super(n);
        this.curChunk = (Object[])new Object[1 << this.initialChunkPower];
    }
    
    SpinedBuffer() {
        this.curChunk = (Object[])new Object[1 << this.initialChunkPower];
    }
    
    protected long capacity() {
        return (this.spineIndex == 0) ? ((long)this.curChunk.length) : (this.priorElementCount[this.spineIndex] + (long)this.spine[this.spineIndex].length);
    }
    
    private void inflateSpine() {
        if (this.spine == null) {
            this.spine = (Object[][])new Object[8][];
            this.priorElementCount = new long[8];
            this.spine[0] = this.curChunk;
        }
    }
    
    protected final void ensureCapacity(long n) {
        long capacity = this.capacity();
        if (n > capacity) {
            this.inflateSpine();
            int chunkSize;
            for (int n2 = this.spineIndex + 1; n > capacity; chunkSize = this.chunkSize(n2), this.spine[n2] = (Object[])new Object[chunkSize], this.priorElementCount[n2] = this.priorElementCount[n2 - 1] + (long)this.spine[n2 - 1].length, capacity += (long)chunkSize, ++n2) {
                if (n2 >= this.spine.length) {
                    int n3 = this.spine.length * 2;
                    this.spine = (E[][])Arrays.copyOf(this.spine, n3);
                    this.priorElementCount = Arrays.copyOf(this.priorElementCount, n3);
                }
            }
        }
    }
    
    protected void increaseCapacity() {
        this.ensureCapacity(this.capacity() + 1L);
    }
    
    public E get(long i) {
        if (this.spineIndex == 0) {
            if (i < (long)this.elementIndex) {
                return this.curChunk[(int)i];
            }
            throw new IndexOutOfBoundsException(Long.toString(i));
        }
        else {
            if (i >= this.count()) {
                throw new IndexOutOfBoundsException(Long.toString(i));
            }
            int j = 0;
            while (j <= this.spineIndex) {
                if (i < this.priorElementCount[j] + (long)this.spine[j].length) {
                    return this.spine[j][(int)(i - this.priorElementCount[j])];
                }
                ++j;
            }
            throw new IndexOutOfBoundsException(Long.toString(i));
        }
    }
    
    public void copyInto(E[] array, int n) {
        long n2 = (long)n + this.count();
        if (n2 > (long)array.length || n2 < (long)n) {
            throw new IndexOutOfBoundsException("does not fit");
        }
        if (this.spineIndex == 0) {
            System.arraycopy(this.curChunk, 0, array, n, this.elementIndex);
        }
        else {
            for (int i = 0; i < this.spineIndex; n += this.spine[i].length, ++i) {
                System.arraycopy(this.spine[i], 0, array, n, this.spine[i].length);
            }
            if (this.elementIndex > 0) {
                System.arraycopy(this.curChunk, 0, array, n, this.elementIndex);
            }
        }
        return;
    }
    
    public E[] asArray(IntFunction<E[]> intFunction) {
        long count = this.count();
        if (count >= 2147483639L) {
            throw new IllegalArgumentException("Stream size exceeds max array size");
        }
        Object[] array = (E[])intFunction.apply((int)count);
        this.copyInto(array, 0);
        return array;
    }
    
    @Override
    public void clear() {
        if (this.spine != null) {
            this.curChunk = this.spine[0];
            for (int j = 0; j < this.curChunk.length; ++j) {
                this.curChunk[j] = null;
            }
            this.spine = (E[][])null;
            this.priorElementCount = null;
        }
        else {
            for (int i = 0; i < this.elementIndex; ++i) {
                this.curChunk[i] = null;
            }
        }
        this.elementIndex = 0;
        this.spineIndex = 0;
    }
    
    @Override
    public Iterator<E> iterator() {
        return Spliterators.iterator(this.spliterator());
    }
    
    @Override
    public void forEach(Consumer<? super E> consumer) {
        for (int i = 0; i < this.spineIndex; ++i) {
            Object[] array = this.spine[i];
            for (int length = array.length, j = 0; j < length; ++j) {
                consumer.accept(array[j]);
            }
        }
        for (int k = 0; k < this.elementIndex; ++k) {
            consumer.accept(this.curChunk[k]);
        }
    }
    
    @Override
    public void accept(E e) {
        if (this.elementIndex == this.curChunk.length) {
            this.inflateSpine();
            if (this.spineIndex + 1 >= this.spine.length || this.spine[this.spineIndex + 1] == null) {
                this.increaseCapacity();
            }
            this.elementIndex = 0;
            ++this.spineIndex;
            this.curChunk = this.spine[this.spineIndex];
        }
        this.curChunk[this.elementIndex++] = e;
    }
    
    @Override
    public String toString() {
        ArrayList<Object> list = new ArrayList();
        this.forEach(list::add);
        return "SpinedBuffer:" + list.toString();
    }
    
    @Override
    public Spliterator<E> spliterator() {
        class Splitr implements Spliterator<E>
        {
            int splSpineIndex = 0;
            final int lastSpineIndex = SpinedBuffer.this.spineIndex;
            int splElementIndex = 0;
            final int lastSpineElementFence = SpinedBuffer.this.elementIndex;
            E[] splChunk;
            
            Splitr(int lastSpineIndex, int splElementIndex, int lastSpineElementFence) {
                assert splSpineIndex == 0 && lastSpineIndex == 0;
                this.splChunk = ((SpinedBuffer.this.spine == null) ? SpinedBuffer.this.curChunk : SpinedBuffer.this.spine[splSpineIndex]);
                return;
            }
            
            @Override
            public long estimateSize() {
                return (this.splSpineIndex == this.lastSpineIndex) ? ((long)this.lastSpineElementFence - (long)this.splElementIndex) : (SpinedBuffer.this.priorElementCount[this.lastSpineIndex] + (long)this.lastSpineElementFence - SpinedBuffer.this.priorElementCount[this.splSpineIndex] - (long)this.splElementIndex);
            }
            
            @Override
            public int characteristics() {
                return 16464;
            }
            
            @Override
            public boolean tryAdvance(Consumer<? super E> obj) {
                Objects.requireNonNull(obj);
                if (this.splSpineIndex < this.lastSpineIndex || (this.splSpineIndex == this.lastSpineIndex && this.splElementIndex < this.lastSpineElementFence)) {
                    obj.accept(this.splChunk[this.splElementIndex++]);
                    if (this.splElementIndex == this.splChunk.length) {
                        this.splElementIndex = 0;
                        ++this.splSpineIndex;
                        if (SpinedBuffer.this.spine != null && this.splSpineIndex <= this.lastSpineIndex) {
                            this.splChunk = SpinedBuffer.this.spine[this.splSpineIndex];
                        }
                    }
                    return true;
                }
                else {
                    return false;
                }
            }
            
            @Override
            public void forEachRemaining(Consumer<? super E> obj) {
                Objects.requireNonNull(obj);
                if (this.splSpineIndex < this.lastSpineIndex || (this.splSpineIndex == this.lastSpineIndex && this.splElementIndex < this.lastSpineElementFence)) {
                    int i = this.splElementIndex;
                    for (int j = this.splSpineIndex; j < this.lastSpineIndex; i = 0, ++j) {
                        for (Object[] array = SpinedBuffer.this.spine[j]; i < array.length; ++i) {
                            obj.accept(array[i]);
                        }
                    }
                    Object[] array2 = (this.splSpineIndex == this.lastSpineIndex) ? this.splChunk : SpinedBuffer.this.spine[this.lastSpineIndex];
                    while (i < this.lastSpineElementFence) {
                        obj.accept(array2[i]);
                        ++i;
                    }
                    this.splSpineIndex = this.lastSpineIndex;
                    this.splElementIndex = this.lastSpineElementFence;
                }
            }
            
            @Override
            public Spliterator<E> trySplit() {
                if (this.splSpineIndex < this.lastSpineIndex) {
                    SpinedBuffer.1Splitr splitr = new Splitr(this.splSpineIndex, this.lastSpineIndex - 1, this.splElementIndex, SpinedBuffer.this.spine[this.lastSpineIndex - 1].length);
                    this.splSpineIndex = this.lastSpineIndex;
                    this.splElementIndex = 0;
                    this.splChunk = SpinedBuffer.this.spine[this.splSpineIndex];
                    return splitr;
                }
                if (this.splSpineIndex == this.lastSpineIndex) {
                    int n = (this.lastSpineElementFence - this.splElementIndex) / 2;
                    if (n == 0) {
                        return null;
                    }
                    Spliterator<E> spliterator = Arrays.spliterator(this.splChunk, this.splElementIndex, this.splElementIndex + n);
                    this.splElementIndex += n;
                    return spliterator;
                }
                else {
                    return null;
                }
            }
        }
        return new Splitr(SpinedBuffer.this.spineIndex, SpinedBuffer.this.elementIndex);
    }
    
    abstract static class OfPrimitive<E, T_ARR, T_CONS> extends AbstractSpinedBuffer implements Iterable<E>
    {
        T_ARR curChunk;
        T_ARR[] spine;
        
        OfPrimitive(int n) {
            super(n);
            this.curChunk = this.newArray(1 << this.initialChunkPower);
        }
        
        OfPrimitive() {
            this.curChunk = this.newArray(1 << this.initialChunkPower);
        }
        
        @Override
        public abstract Iterator<E> iterator();
        
        @Override
        public abstract void forEach(Consumer<? super E> p0);
        
        protected abstract T_ARR[] newArrayArray(int p0);
        
        public abstract T_ARR newArray(int p0);
        
        protected abstract int arrayLength(T_ARR p0);
        
        protected abstract void arrayForEach(T_ARR p0, int p1, int p2, T_CONS p3);
        
        protected long capacity() {
            return (this.spineIndex == 0) ? ((long)this.arrayLength(this.curChunk)) : (this.priorElementCount[this.spineIndex] + (long)this.arrayLength(this.spine[this.spineIndex]));
        }
        
        private void inflateSpine() {
            if (this.spine == null) {
                this.spine = this.newArrayArray(8);
                this.priorElementCount = new long[8];
                this.spine[0] = this.curChunk;
            }
        }
        
        protected final void ensureCapacity(long n) {
            long capacity = this.capacity();
            if (n > capacity) {
                this.inflateSpine();
                int chunkSize;
                for (int n2 = this.spineIndex + 1; n > capacity; chunkSize = this.chunkSize(n2), this.spine[n2] = this.newArray(chunkSize), this.priorElementCount[n2] = this.priorElementCount[n2 - 1] + (long)this.arrayLength(this.spine[n2 - 1]), capacity += (long)chunkSize, ++n2) {
                    if (n2 >= this.spine.length) {
                        int n3 = this.spine.length * 2;
                        this.spine = Arrays.copyOf(this.spine, n3);
                        this.priorElementCount = Arrays.copyOf(this.priorElementCount, n3);
                    }
                }
            }
        }
        
        protected void increaseCapacity() {
            this.ensureCapacity(this.capacity() + 1L);
        }
        
        protected int chunkFor(long i) {
            if (this.spineIndex == 0) {
                if (i < (long)this.elementIndex) {
                    return 0;
                }
                throw new IndexOutOfBoundsException(Long.toString(i));
            }
            else {
                if (i >= this.count()) {
                    throw new IndexOutOfBoundsException(Long.toString(i));
                }
                int j = 0;
                while (j <= this.spineIndex) {
                    if (i < this.priorElementCount[j] + (long)this.arrayLength(this.spine[j])) {
                        return j;
                    }
                    ++j;
                }
                throw new IndexOutOfBoundsException(Long.toString(i));
            }
        }
        
        public void copyInto(T_ARR t_ARR, int n) {
            long n2 = (long)n + this.count();
            if (n2 > (long)this.arrayLength(t_ARR) || n2 < (long)n) {
                throw new IndexOutOfBoundsException("does not fit");
            }
            if (this.spineIndex == 0) {
                System.arraycopy(this.curChunk, 0, t_ARR, n, this.elementIndex);
            }
            else {
                for (int i = 0; i < this.spineIndex; n += this.arrayLength(this.spine[i]), ++i) {
                    System.arraycopy(this.spine[i], 0, t_ARR, n, this.arrayLength(this.spine[i]));
                }
                if (this.elementIndex > 0) {
                    System.arraycopy(this.curChunk, 0, t_ARR, n, this.elementIndex);
                }
            }
            return;
        }
        
        public T_ARR asPrimitiveArray() {
            long count = this.count();
            if (count >= 2147483639L) {
                throw new IllegalArgumentException("Stream size exceeds max array size");
            }
            Object array = this.newArray((int)count);
            this.copyInto(array, 0);
            return array;
        }
        
        protected void preAccept() {
            if (this.elementIndex == this.arrayLength(this.curChunk)) {
                this.inflateSpine();
                if (this.spineIndex + 1 >= this.spine.length || this.spine[this.spineIndex + 1] == null) {
                    this.increaseCapacity();
                }
                this.elementIndex = 0;
                ++this.spineIndex;
                this.curChunk = this.spine[this.spineIndex];
            }
        }
        
        @Override
        public void clear() {
            if (this.spine != null) {
                this.curChunk = this.spine[0];
                this.spine = null;
                this.priorElementCount = null;
            }
            this.elementIndex = 0;
            this.spineIndex = 0;
        }
        
        public void forEach(T_CONS t_CONS) {
            for (int i = 0; i < this.spineIndex; ++i) {
                this.arrayForEach(this.spine[i], 0, this.arrayLength(this.spine[i]), t_CONS);
            }
            this.arrayForEach(this.curChunk, 0, this.elementIndex, t_CONS);
        }
        
        abstract class BaseSpliterator<T_SPLITR extends Spliterator.OfPrimitive<E, T_CONS, T_SPLITR>> implements Spliterator.OfPrimitive<E, T_CONS, T_SPLITR>
        {
            int splSpineIndex;
            final int lastSpineIndex;
            int splElementIndex;
            final int lastSpineElementFence;
            T_ARR splChunk;
            
            BaseSpliterator(int splSpineIndex, int lastSpineIndex, int splElementIndex, int lastSpineElementFence) {
                this.splSpineIndex = splSpineIndex;
                this.lastSpineIndex = lastSpineIndex;
                this.splElementIndex = splElementIndex;
                this.lastSpineElementFence = lastSpineElementFence;
                assert splSpineIndex == 0 && lastSpineIndex == 0;
                this.splChunk = ((SpinedBuffer.OfPrimitive.this.spine == null) ? SpinedBuffer.OfPrimitive.this.curChunk : SpinedBuffer.OfPrimitive.this.spine[splSpineIndex]);
                return;
            }
            
            abstract T_SPLITR newSpliterator(int p0, int p1, int p2, int p3);
            
            abstract void arrayForOne(T_ARR p0, int p1, T_CONS p2);
            
            abstract T_SPLITR arraySpliterator(T_ARR p0, int p1, int p2);
            
            @Override
            public long estimateSize() {
                return (this.splSpineIndex == this.lastSpineIndex) ? ((long)this.lastSpineElementFence - (long)this.splElementIndex) : (SpinedBuffer.OfPrimitive.this.priorElementCount[this.lastSpineIndex] + (long)this.lastSpineElementFence - SpinedBuffer.OfPrimitive.this.priorElementCount[this.splSpineIndex] - (long)this.splElementIndex);
            }
            
            @Override
            public int characteristics() {
                return 16464;
            }
            
            @Override
            public boolean tryAdvance(T_CONS obj) {
                Objects.requireNonNull(obj);
                if (this.splSpineIndex < this.lastSpineIndex || (this.splSpineIndex == this.lastSpineIndex && this.splElementIndex < this.lastSpineElementFence)) {
                    this.arrayForOne(this.splChunk, this.splElementIndex++, obj);
                    if (this.splElementIndex == SpinedBuffer.OfPrimitive.this.arrayLength(this.splChunk)) {
                        this.splElementIndex = 0;
                        ++this.splSpineIndex;
                        if (SpinedBuffer.OfPrimitive.this.spine != null && this.splSpineIndex <= this.lastSpineIndex) {
                            this.splChunk = SpinedBuffer.OfPrimitive.this.spine[this.splSpineIndex];
                        }
                    }
                    return true;
                }
                else {
                    return false;
                }
            }
            
            @Override
            public void forEachRemaining(T_CONS obj) {
                Objects.requireNonNull(obj);
                if (this.splSpineIndex < this.lastSpineIndex || (this.splSpineIndex == this.lastSpineIndex && this.splElementIndex < this.lastSpineElementFence)) {
                    int splElementIndex = this.splElementIndex;
                    for (int i = this.splSpineIndex; i < this.lastSpineIndex; splElementIndex = 0, ++i) {
                        Object o = SpinedBuffer.OfPrimitive.this.spine[i];
                        SpinedBuffer.OfPrimitive.this.arrayForEach(o, splElementIndex, SpinedBuffer.OfPrimitive.this.arrayLength(o), obj);
                    }
                    SpinedBuffer.OfPrimitive.this.arrayForEach((this.splSpineIndex == this.lastSpineIndex) ? this.splChunk : SpinedBuffer.OfPrimitive.this.spine[this.lastSpineIndex], splElementIndex, this.lastSpineElementFence, obj);
                    this.splSpineIndex = this.lastSpineIndex;
                    this.splElementIndex = this.lastSpineElementFence;
                }
            }
            
            @Override
            public T_SPLITR trySplit() {
                if (this.splSpineIndex < this.lastSpineIndex) {
                    Spliterator.OfPrimitive<E, T_CONS, Spliterator.OfPrimitive<E, T_CONS, T_SPLITR>> spliterator = this.newSpliterator(this.splSpineIndex, this.lastSpineIndex - 1, this.splElementIndex, SpinedBuffer.OfPrimitive.this.arrayLength(SpinedBuffer.OfPrimitive.this.spine[this.lastSpineIndex - 1]));
                    this.splSpineIndex = this.lastSpineIndex;
                    this.splElementIndex = 0;
                    this.splChunk = SpinedBuffer.OfPrimitive.this.spine[this.splSpineIndex];
                    return spliterator;
                }
                if (this.splSpineIndex == this.lastSpineIndex) {
                    int n = (this.lastSpineElementFence - this.splElementIndex) / 2;
                    if (n == 0) {
                        return null;
                    }
                    Spliterator.OfPrimitive<E, T_CONS, Spliterator.OfPrimitive<E, T_CONS, T_SPLITR>> arraySpliterator = this.arraySpliterator(this.splChunk, this.splElementIndex, n);
                    this.splElementIndex += n;
                    return arraySpliterator;
                }
                else {
                    return null;
                }
            }
        }
    }
    
    static class OfInt extends SpinedBuffer.OfPrimitive<Integer, int[], IntConsumer> implements IntConsumer
    {
        OfInt() {
        }
        
        OfInt(int n) {
            super(n);
        }
        
        @Override
        public void forEach(Consumer<? super Integer> action) {
            if ((action instanceof IntConsumer)) {
                super.forEach((IntConsumer)action);
            }
            else {
                if (Tripwire.ENABLED) {
                    Tripwire.trip(this.getClass(), "{0} calling SpinedBuffer.OfInt.forEach(Consumer)");
                }
                this.spliterator().forEachRemaining(action);
            }
        }
        
        @Override
        protected int[][] newArrayArray(int n) {
            return new int[n][];
        }
        
        @Override
        public int[] newArray(int n) {
            return new int[n];
        }
        
        @Override
        protected int arrayLength(int[] array) {
            return array.length;
        }
        
        @Override
        protected void arrayForEach(int[] array, int i, int n, IntConsumer intConsumer) {
            for (; i < n; ++i) {
                intConsumer.accept(array[i]);
            }
        }
        
        @Override
        public void accept(int n) {
            this.preAccept();
            ((int[])this.curChunk)[this.elementIndex++] = n;
        }
        
        public int get(long n) {
            int chunk = this.chunkFor(n);
            if (this.spineIndex == 0 && chunk == 0) {
                return ((int[])this.curChunk)[(int)n];
            }
            return ((int[][])this.spine)[chunk][(int)(n - this.priorElementCount[chunk])];
        }
        
        @Override
        public PrimitiveIterator.OfInt iterator() {
            return Spliterators.iterator(this.spliterator());
        }
        
        @Override
        public Spliterator.OfInt spliterator() {
            class Splitr extends SpinedBuffer.OfPrimitive.BaseSpliterator<Spliterator.OfInt> implements Spliterator.OfInt
            {
                Splitr(int n2, int n3, int n4) {
                    super(n, n2, n3, n4);
                }
                
                @Override
                SpinedBuffer.OfInt.1Splitr newSpliterator(int n, int n2, int n3, int n4) {
                    return new Splitr(n, n2, n3, n4);
                }
                
                void arrayForOne(int[] array, int n, IntConsumer intConsumer) {
                    intConsumer.accept(array[n]);
                }
                
                Spliterator.OfInt arraySpliterator(int[] array, int startInclusive, int n) {
                    return Arrays.spliterator(array, startInclusive, startInclusive + n);
                }
            }
            return new Splitr(0, this.spineIndex, 0, this.elementIndex);
        }
        
        @Override
        public String toString() {
            int[] array = (int[])this.asPrimitiveArray();
            if (array.length < 200) {
                return String.format("%s[length=%d, chunks=%d]%s", new Object[] { this.getClass().getSimpleName(), Integer.valueOf(array.length), Integer.valueOf(this.spineIndex), Arrays.toString(array) });
            }
            return String.format("%s[length=%d, chunks=%d]%s...", new Object[] { this.getClass().getSimpleName(), Integer.valueOf(array.length), Integer.valueOf(this.spineIndex), Arrays.toString(Arrays.copyOf(array, 200)) });
        }
    }
    
    static class OfLong extends SpinedBuffer.OfPrimitive<Long, long[], LongConsumer> implements LongConsumer
    {
        OfLong() {
        }
        
        OfLong(int n) {
            super(n);
        }
        
        @Override
        public void forEach(Consumer<? super Long> action) {
            if ((action instanceof LongConsumer)) {
                super.forEach((LongConsumer)action);
            }
            else {
                if (Tripwire.ENABLED) {
                    Tripwire.trip(this.getClass(), "{0} calling SpinedBuffer.OfLong.forEach(Consumer)");
                }
                this.spliterator().forEachRemaining(action);
            }
        }
        
        @Override
        protected long[][] newArrayArray(int n) {
            return new long[n][];
        }
        
        @Override
        public long[] newArray(int n) {
            return new long[n];
        }
        
        @Override
        protected int arrayLength(long[] array) {
            return array.length;
        }
        
        @Override
        protected void arrayForEach(long[] array, int i, int n, LongConsumer longConsumer) {
            for (; i < n; ++i) {
                longConsumer.accept(array[i]);
            }
        }
        
        @Override
        public void accept(long n) {
            this.preAccept();
            ((long[])this.curChunk)[this.elementIndex++] = n;
        }
        
        public long get(long n) {
            int chunk = this.chunkFor(n);
            if (this.spineIndex == 0 && chunk == 0) {
                return ((long[])this.curChunk)[(int)n];
            }
            return ((long[][])this.spine)[chunk][(int)(n - this.priorElementCount[chunk])];
        }
        
        @Override
        public PrimitiveIterator.OfLong iterator() {
            return Spliterators.iterator(this.spliterator());
        }
        
        @Override
        public Spliterator.OfLong spliterator() {
            class Splitr extends SpinedBuffer.OfPrimitive.BaseSpliterator<Spliterator.OfLong> implements Spliterator.OfLong
            {
                Splitr(int n2, int n3, int n4) {
                    super(n, n2, n3, n4);
                }
                
                @Override
                SpinedBuffer.OfLong.1Splitr newSpliterator(int n, int n2, int n3, int n4) {
                    return new Splitr(n, n2, n3, n4);
                }
                
                void arrayForOne(long[] array, int n, LongConsumer longConsumer) {
                    longConsumer.accept(array[n]);
                }
                
                Spliterator.OfLong arraySpliterator(long[] array, int startInclusive, int n) {
                    return Arrays.spliterator(array, startInclusive, startInclusive + n);
                }
            }
            return new Splitr(0, this.spineIndex, 0, this.elementIndex);
        }
        
        @Override
        public String toString() {
            long[] array = (long[])this.asPrimitiveArray();
            if (array.length < 200) {
                return String.format("%s[length=%d, chunks=%d]%s", new Object[] { this.getClass().getSimpleName(), Integer.valueOf(array.length), Integer.valueOf(this.spineIndex), Arrays.toString(array) });
            }
            return String.format("%s[length=%d, chunks=%d]%s...", new Object[] { this.getClass().getSimpleName(), Integer.valueOf(array.length), Integer.valueOf(this.spineIndex), Arrays.toString(Arrays.copyOf(array, 200)) });
        }
    }
    
    static class OfDouble extends SpinedBuffer.OfPrimitive<Double, double[], DoubleConsumer> implements DoubleConsumer
    {
        OfDouble() {
        }
        
        OfDouble(int n) {
            super(n);
        }
        
        @Override
        public void forEach(Consumer<? super Double> action) {
            if ((action instanceof DoubleConsumer)) {
                super.forEach((DoubleConsumer)action);
            }
            else {
                if (Tripwire.ENABLED) {
                    Tripwire.trip(this.getClass(), "{0} calling SpinedBuffer.OfDouble.forEach(Consumer)");
                }
                this.spliterator().forEachRemaining(action);
            }
        }
        
        @Override
        protected double[][] newArrayArray(int n) {
            return new double[n][];
        }
        
        @Override
        public double[] newArray(int n) {
            return new double[n];
        }
        
        @Override
        protected int arrayLength(double[] array) {
            return array.length;
        }
        
        @Override
        protected void arrayForEach(double[] array, int i, int n, DoubleConsumer doubleConsumer) {
            for (; i < n; ++i) {
                doubleConsumer.accept(array[i]);
            }
        }
        
        @Override
        public void accept(double n) {
            this.preAccept();
            ((double[])this.curChunk)[this.elementIndex++] = n;
        }
        
        public double get(long n) {
            int chunk = this.chunkFor(n);
            if (this.spineIndex == 0 && chunk == 0) {
                return ((double[])this.curChunk)[(int)n];
            }
            return ((double[][])this.spine)[chunk][(int)(n - this.priorElementCount[chunk])];
        }
        
        @Override
        public PrimitiveIterator.OfDouble iterator() {
            return Spliterators.iterator(this.spliterator());
        }
        
        @Override
        public Spliterator.OfDouble spliterator() {
            class Splitr extends SpinedBuffer.OfPrimitive.BaseSpliterator<Spliterator.OfDouble> implements Spliterator.OfDouble
            {
                Splitr(int n2, int n3, int n4) {
                    super(n, n2, n3, n4);
                }
                
                @Override
                SpinedBuffer.OfDouble.1Splitr newSpliterator(int n, int n2, int n3, int n4) {
                    return new Splitr(n, n2, n3, n4);
                }
                
                void arrayForOne(double[] array, int n, DoubleConsumer doubleConsumer) {
                    doubleConsumer.accept(array[n]);
                }
                
                Spliterator.OfDouble arraySpliterator(double[] array, int startInclusive, int n) {
                    return Arrays.spliterator(array, startInclusive, startInclusive + n);
                }
            }
            return new Splitr(0, this.spineIndex, 0, this.elementIndex);
        }
        
        @Override
        public String toString() {
            double[] array = (double[])this.asPrimitiveArray();
            if (array.length < 200) {
                return String.format("%s[length=%d, chunks=%d]%s", new Object[] { this.getClass().getSimpleName(), Integer.valueOf(array.length), Integer.valueOf(this.spineIndex), Arrays.toString(array) });
            }
            return String.format("%s[length=%d, chunks=%d]%s...", new Object[] { this.getClass().getSimpleName(), Integer.valueOf(array.length), Integer.valueOf(this.spineIndex), Arrays.toString(Arrays.copyOf(array, 200)) });
        }
    }
}
